/*
 *  tcpinject v0.1
 *
 *  Copyright (c) 2000, 2001 Marc Kirchner <kirchner@stud.fh-heilbronn.de>
 *
 *
 *  (formerly) libnet example code by:
 *  Copyright (c) 1999, 2000 Mike D. Schiffman <mike@infonexus.com>
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <libnet.h>
#include <stdio.h>

void usage(char *);

int main(int argc, char **argv) {

	int 	network, packet_size=0,
		c=0, content_size=0, 
		win_size=1024, ttl=48, i=0;
	u_long 	src_ip=0, dst_ip=0,
		packetcount=1, count=0;
	u_short src_prt=0, dst_prt=0;
	u_char 	*cp, *packet, *content, 
		cntlflag=TH_SYN, chr='\0';
	char 	*ch="", *path="";
	FILE 	*fd=NULL;


	while((c = getopt(argc, argv, "c:d:f::l:n:s:w:z:")) != EOF) {      
		switch (c) {
		    case 'c':
			printf ("optarg is %s\n",optarg);
			if (ch=strstr(optarg,"URG")) cntlflag = TH_URG;
			else if (ch=strstr(optarg,"ACK")) cntlflag = TH_ACK;
			else if (ch=strstr(optarg,"PSH")) cntlflag = TH_PUSH;
			else if (ch=strstr(optarg,"RST")) cntlflag = TH_RST;
			else if (ch=strstr(optarg,"SYN")) cntlflag = TH_SYN;
			else if (ch=strstr(optarg,"FIN")) cntlflag = TH_FIN;
			printf("setting control flag to %#x\n", cntlflag);
			break;
		    case 'd':
			if (!(cp = strrchr(optarg, '.'))) {
				usage(argv[0]);
			}
			*cp++ = 0;
			dst_prt = (u_short)atoi(cp);
			if (!(dst_ip = libnet_name_resolve(optarg, LIBNET_RESOLVE))) {
				libnet_error(LIBNET_ERR_FATAL, "bad destination IP address %s\n", optarg);
			}
			break;
		    case 'f':
			path = optarg;
			if ((path) && !(fd=fopen(path, "r"))) {
				printf("couldn't find %s\n", optarg);
				exit;
			} else if (!(path)) { 
				fd=stdin;
				printf("Reading from stdin...end with Ctrl-D\n");
			}
			break;
		    case 'l':
			ttl = (int)atoi(optarg);
	                break;
		    case 'n':
			packetcount = (int)atoi(optarg);
			break;
		    case 's':
			if (!(cp = strrchr(optarg, '.'))) {
				usage(argv[0]);
			}
			*cp++ = 0;
			src_prt = (u_short)atoi(cp);
			if (!(src_ip = libnet_name_resolve(optarg, LIBNET_RESOLVE))) {
				libnet_error(LIBNET_ERR_FATAL, "bad source IP address %s\n", optarg);
			}
			break;
		    case 'w':
			win_size = (int)atoi(optarg);
			break;
		    case 'z':
			content_size = (int)atoi(optarg);
			break;
		} /* switch */
	} /* while */

	/* check if we've got enough info to start */
	if (!src_ip || !src_prt || !dst_ip || !dst_prt) {
		usage(argv[0]);
		exit(EXIT_FAILURE);
	}

	/* initialize content memory */
	content=malloc(content_size);
	if (content) {
		bzero(content, content_size);
	} else {
		fprintf(stderr, "Could not get enough memory. malloc failed.");
	}
	if (fd) {
		for (i=0; i<content_size; i++) {
			if ((chr=(u_char)fgetc(fd)) > 0) *(content+i)=chr;
		}
	}

	/* initialize packet mem */
	packet_size = LIBNET_IP_H + LIBNET_TCP_H + content_size;
	libnet_init_packet(packet_size, &packet);
	if (packet == NULL) {
		libnet_error(LIBNET_ERR_FATAL, "libnet_init_packet failed\n");
	}

	/* Network initialization */
	network = libnet_open_raw_sock(IPPROTO_RAW);
	if (network == -1) {
		libnet_error(LIBNET_ERR_FATAL, "Can't open network.\n");
	}
    
	/* IP header */
	libnet_build_ip(LIBNET_TCP_H+content_size,   /* size of the packet w/o IP header */
			IPTOS_LOWDELAY,         	/* IP tos */
			242,                    	/* IP ID */
			0,                      	/* frag stuff */
			ttl,                     	/* TTL */
			IPPROTO_TCP,            	/* transport protocol */
			src_ip,                 	/* source IP */
			dst_ip,                 	/* destination IP */
			(const u_char*) &content,   /* payload */
			(int) content_size,         /* payload length */
			packet);                	/* packet header memory */

	/* TCP header */
	libnet_build_tcp(src_prt,       	/* source TCP port */
			dst_prt,                	/* destination TCP port */
			0xa1d95,                	/* sequence number */
			0x53,                   	/* acknowledgement number */
			cntlflag,                 	/* control flags */
			win_size,                  	/* window size */
			0,                      	/* urgent pointer */
			content,                   	/* payload */
			(int) content_size,         /* payload length */
			packet + LIBNET_IP_H);  	/* packet header memory */

	/* checksums (TCP header only) */
	if (libnet_do_checksum(packet, IPPROTO_TCP, LIBNET_TCP_H) == -1) {
		libnet_error(LIBNET_ERR_FATAL, "libnet_do_checksum failed\n");
	}

	/* let's rock */
	for (count=0; count<packetcount; count++) {
		c = libnet_write_ip(network, packet, packet_size);
		if (c < packet_size) {
			libnet_error(LN_ERR_WARNING, "packet #%d: libnet_write_ip only wrote %d bytes\n", count, c);
		} else {
			fprintf(stdout,"packet #%d with size %d bytes\n", count, c);
		}
	}
    
	/* shut down interface */
	if (libnet_close_raw_sock(network) == -1) {
		libnet_error(LN_ERR_WARNING, "libnet_close_raw_sock couldn't close the interface");
	}

	/* free packet memory */
	free(content);
	libnet_destroy_packet(&packet);

	/* determine exit code */
	return (c == -1 ? EXIT_FAILURE : EXIT_SUCCESS);
}


void usage(char *name) {
fprintf(stderr, "usage: %s -s s_ip.s_port -d d_ip.d_port -z packet_size \n\t-c TCP_control_flag -l TTL -w TCP_window_size\n\t-n packet_count -ffile\n", name);
}
